//
//
// eyeye.c
//
// nadya peek peek@mit.edu sept 2013
// akito van troyer
//
//

#include <avr/io.h>
#include <util/delay.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define button_port PORTA
#define button_direction DDRA
#define button_pins PINA
#define button_pin (1 << PA2)

#define pot_port PORTA
#define pot_direction DDRA
#define pot_pins PINA
#define pot_pin (1 << PA0)

#define led_port PORTB
#define led_direction DDRB
#define led_pin (1 << PB1)

#define speaker_port PORTA
#define speaker_direction DDRA
#define speaker_pin (1 << PA3) 

uint16_t i = 0;
uint16_t rand = 0;
uint8_t counter = 0;
uint8_t maxTableSize = 36;

uint16_t table[] = {
110,
116.5,
123.5,
130.8,
138.6,
146.8,
155.6,
164.8,
174.6,
185,
196,
207.7,
220,
233.1,
246.9,
261.6,
277.2,
293.7,
311.1,
329.6,
349.2,
370,
392,
415.3,
440,
466.2,
493.9,
523.3,
554.4,
587.3,
622.3,
659.3,
698.5,
740,
784,
830.6,
880
};

uint8_t get_random8(uint8_t seed) {
   static uint8_t z=1,w=1;
   z = 269 * (seed & 255) + (seed >> 6);
   w = 180 * (w & 255) + (w >> 6);
   return (z << 8) + w;  /* 8-bit result */
}

uint16_t get_random16(uint16_t seed) {
    static uint16_t z=1,w=1;
    z = 269 * (seed & 65536) + (seed >> 6);
    w = 2560 * (w & 65536) + (w >> 6);
    return (z << 8) + w;  /* 16-bit result */
}

uint32_t midi2freq(uint8_t midi){
    uint16_t temp = 2,i;
    for(i=0;i<midi;i++){
        temp *= 2;
    }
    return temp;
}

int linlin(int x, int in_min, int in_max, int out_min, int out_max) {
    return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}

int main(void) {
   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // init A/D
   //
   ADMUX = (0 << REFS1) | (0 << REFS0) // VCC ref
      | (0 << ADLAR) // right adjust
      | (0 << MUX3) | (0 << MUX2) | (0 << MUX1) | (0 << MUX0); //ADC0
   ADCSRA = (1 << ADEN) // enable
      | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // prescaler /128
   ADCSRB = (1 << BIN); // bipolar mode
   //
   // main loop
   //
   output(led_direction, led_pin);
   input(button_direction, button_pin);
   //set(button_port, button_pin);
   input(pot_direction,pot_pin);

   //TCCR1A = ((1 << COM1B1) | (1 << COM1B0) | (1 << WGM11)
   //   | (1 << WGM10)); // clear OC1B on compare match, fast 8-bit PWM
   //TCCR1B = ((0 << WGM13) | (1 << WGM12) | (0 << CS12) | (0 << CS11)
   //   | (1 << CS10)); // /1 clock divider

   clear(speaker_port,speaker_pin);
   output(speaker_direction,speaker_pin);
   
   while (1) {
      ADCSRA |= (1 << ADSC); // start ADC reading
      while (ADCSRA & (1 << ADSC)); // this takes time

      rand = table[linlin(ADC,0,1024,0,maxTableSize)]; //add some noise

      if(pin_test(button_pins,button_pin) != 0){
          // if the mouth is open, turn on light and start playing
          set(led_port, led_pin);
          set(speaker_port, speaker_pin);
          for(i=0;i < rand;i++){ //who needs timer counters when you only have 20 lines of code
            _delay_us(2);
          }
          clear(speaker_port, speaker_pin);
          for(i=0;i < rand;i++){
            _delay_us(2);
          }
      }
      else {
        clear(led_port, led_pin);
      }
   }
}
